namespace :rdms do
  desc 'Add users to RDMS. usage: rdms:setup_users["users.json", "false"]'
  task :setup_users, [:seedfile, :update_users] => :environment do |task, args|

    seedfile = args.seedfile
    update_user = args.update_users
    if update_user.is_a? String
      if update_user.downcase == 'false'
        update_user = false
      else
        update_user = true
      end
    end
    
    if (File.exists?(seedfile))
      puts("Running task to add / update users from seedfile: #{seedfile}")
    else
      abort("ERROR: missing seedfile for user setup: #{seedfile}")
    end

    seed = JSON.parse(File.read(seedfile))

    seed.fetch('users', []).each_with_index do |parsed_user, index|
      messages = []
      user_count = index + 1
      user = User.find_by_hash(parsed_user)
      if user
        email = user.email
      else
        email = User.find_or_create_user_email(parsed_user)
      end

      if user
        if update_user
          messages << "Info: Updating user #{email}"
        else
          messages << "Warning: User #{email} already exists. Not updating user."
          user = nil
        end
      elsif email.blank?
        messages << "Error: Not creating user #{user_count}. User has no email, saml_id or orcid."
      else
        messages << "Info: Creating user #{email}"
        user = User.new
      end

      if user.present?
        user.email = email

        if parsed_user.fetch('password', nil).present?
          user.password = parsed_user['password']
        elsif user.password.blank?
          user.password = SecureRandom.random_bytes(32)
        end

        user.display_name = parsed_user["name"] if parsed_user.fetch('name', nil).present?

        if parsed_user.fetch('saml_id', nil).present?
          user.provider = "saml"
          user.uid = parsed_user['saml_id']
        end

        user.orcid = parsed_user['orcid'] if parsed_user.fetch('orcid', nil).present?

        if user.save
          _success, message = User.assign_user_to_role(user, parsed_user)
          messages << message
        else
          messages << "Error: #{user.errors.full_messages}"
        end
      end

      if messages.any?
        puts "--- User: #{user_count} ---"
        puts messages
      end
    end
  end

end

# Example json file
# {
#   "users":
#   [{
#     "email": "example_user@hyrax",
#     "password": "password",
#     "name": "Example User",
#     "saml_id": "urn:oasis:names:tc:SAML:attribute:pairwise-id",
#      "orcid": "orcid id",
#     "role": "crc_1280_group_manager",
#     group_id: "vd66vz88c"
#   }]
# }
